@@ -106,11 +106,18 @@ wired_menu_item_activate (GtkMenuItem *item, gpointer user_data)
 	                                  user_data);
 }
 
+
+typedef enum {
+	ADD_ACTIVE = 1,
+	ADD_INACTIVE = 2,
+} AddActiveInactiveEnum;
+
 static void
 add_connection_items (NMDevice *device,
                       GSList *connections,
                       gboolean carrier,
                       NMConnection *active,
+                      AddActiveInactiveEnum flag,
                       GtkWidget *menu,
                       NMApplet *applet)
 {
@@ -119,16 +126,18 @@ add_connection_items (NMDevice *device,
 
 	for (iter = connections; iter; iter = g_slist_next (iter)) {
 		NMConnection *connection = NM_CONNECTION (iter->data);
-		NMSettingConnection *s_con;
 		GtkWidget *item;
 
-		s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
-		item = gtk_check_menu_item_new_with_label (nm_setting_connection_get_id (s_con));
- 		gtk_widget_set_sensitive (GTK_WIDGET (item), carrier);
-		gtk_check_menu_item_set_draw_as_radio (GTK_CHECK_MENU_ITEM (item), TRUE);
+		if (active == connection) {
+			if ((flag & ADD_ACTIVE) == 0)
+				continue;
+		} else {
+			if ((flag & ADD_INACTIVE) == 0)
+				continue;
+		}
 
-		if (connection == active)
-			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), TRUE);
+		item = applet_new_menu_item_helper (connection, active, (flag & ADD_ACTIVE));
+		gtk_widget_set_sensitive (item, carrier);
 
 		info = g_slice_new0 (WiredMenuItemInfo);
 		info->applet = applet;
@@ -180,8 +189,6 @@ wired_add_menu_item (NMDevice *device,
 	GtkWidget *item;
 	GSList *connections, *all;
 	gboolean carrier = TRUE;
-	GtkWidget *label;
-	char *bold_text;
 
 	all = applet_get_all_connections (applet);
 	connections = utils_filter_connections_for_device (device, all);
@@ -206,7 +213,7 @@ wired_add_menu_item (NMDevice *device,
 			text = g_strdup (_("Wired Network"));
 	}
 
-	item = gtk_menu_item_new_with_label (text);
+	item = applet_menu_item_create_device_item_helper (device, applet, text);
 	g_free (text);
 
 	/* Only dim the item if the device supports carrier detection AND
@@ -215,30 +222,30 @@ wired_add_menu_item (NMDevice *device,
  	if (nm_device_get_capabilities (device) & NM_DEVICE_CAP_CARRIER_DETECT)
 		carrier = nm_device_ethernet_get_carrier (NM_DEVICE_ETHERNET (device));
 
-	label = gtk_bin_get_child (GTK_BIN (item));
-	bold_text = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
-	                                     gtk_label_get_text (GTK_LABEL (label)));
-	gtk_label_set_markup (GTK_LABEL (label), bold_text);
-	g_free (bold_text);
-
 	gtk_widget_set_sensitive (item, FALSE);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
+	if (g_slist_length (connections))
+		add_connection_items (device, connections, carrier, active, ADD_ACTIVE, menu, applet);
+
 	/* Notify user of unmanaged or unavailable device */
-	item = nma_menu_device_check_unusable (device, carrier ? NULL : _("disconnected"));
+	item = nma_menu_device_get_menu_item (device, applet, carrier ? NULL : _("disconnected"));
 	if (item) {
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
-		goto out;
 	}
 
-	if (g_slist_length (connections))
-		add_connection_items (device, connections, carrier, active, menu, applet);
-	else
-		add_default_connection_item (device, carrier, menu, applet);
+	if (!nma_menu_device_check_unusable (device)) {
+		if ((!active && g_slist_length (connections)) || (active && g_slist_length (connections) > 1))
+			applet_menu_item_add_complex_separator_helper (menu, applet, _("Available"), -1);
+
+		if (g_slist_length (connections))
+			add_connection_items (device, connections, carrier, active, ADD_INACTIVE, menu, applet);
+		else
+			add_default_connection_item (device, carrier, menu, applet);
+	}
 
-out:
 	g_slist_free (connections);
 }
 
@@ -303,7 +310,7 @@ wired_get_icon (NMDevice *device,
 		*tip = g_strdup_printf (_("Requesting a wired network address for '%s'..."), id);
 		break;
 	case NM_DEVICE_STATE_ACTIVATED:
-		pixbuf = applet->wired_icon;
+		pixbuf = nma_icon_check_and_load ("nm-device-wired", &applet->wired_icon, applet);
 		*tip = g_strdup_printf (_("Wired network connection '%s' active"), id);
 		break;
 	default:
@@ -322,8 +329,9 @@ typedef struct {
 	GtkWidget *ok_button;
 
 	NMApplet *applet;
-	NMConnection *connection;
-	DBusGMethodInvocation *context;
+	NMSettingsConnectionInterface *connection;
+	NMANewSecretsRequestedFunc callback;
+	gpointer callback_data;
 
 	GtkWidget *dialog;
 	NMActiveConnection *active_connection;
@@ -388,31 +396,14 @@ pppoe_update_ui (NMConnection *connection, NMPppoeInfo *info)
 	s = nm_setting_pppoe_get_password (s_pppoe);
 	if (s)
 		gtk_entry_set_text (info->password_entry, s);
-	else {
-		GHashTable *secrets;
-		GError *error = NULL;
-		GValue *value;
-
-		/* Grab password from keyring if possible */
-		secrets = nm_gconf_get_keyring_items (connection,
-		                                      nm_setting_get_name (NM_SETTING (s_pppoe)),
-		                                      FALSE,
-		                                      &error);
-		if (secrets) {
-			value = g_hash_table_lookup (secrets, NM_SETTING_PPPOE_PASSWORD);
-			if (value)
-				gtk_entry_set_text (info->password_entry, g_value_get_string (value));
-			g_hash_table_destroy (secrets);
-		} else if (error)
-			g_error_free (error);
-	}
 }
 
 static NMPppoeInfo *
 pppoe_info_new (GladeXML *xml,
                 NMApplet *applet,
-                DBusGMethodInvocation *context,
-                NMConnection *connection,
+				NMANewSecretsRequestedFunc callback,
+				gpointer callback_data,
+                NMSettingsConnectionInterface *connection,
                 NMActiveConnection *active_connection)
 {
 	NMPppoeInfo *info;
@@ -428,7 +419,8 @@ pppoe_info_new (GladeXML *xml,
 	g_signal_connect (info->password_entry, "changed", G_CALLBACK (pppoe_verify), info);
 
 	info->applet = applet;
-	info->context = context;
+	info->callback = callback;
+	info->callback_data = callback_data;
 	info->connection = g_object_ref (connection);
 	info->active_connection = active_connection;
 
@@ -457,17 +449,25 @@ destroy_pppoe_dialog (gpointer data, GObject *finalized)
 	pppoe_info_destroy (info, NULL);
 }
 
+static void
+update_cb (NMSettingsConnectionInterface *connection,
+           GError *error,
+           gpointer user_data)
+{
+	if (error)
+		g_warning ("Error saving connection secrets: (%d) %s", error->code, error->message);
+}
+
 static void
 get_pppoe_secrets_cb (GtkDialog *dialog,
 					  gint response,
 					  gpointer user_data)
 {
 	NMPppoeInfo *info = (NMPppoeInfo *) user_data;
-	NMAGConfConnection *gconf_connection;
 	NMSetting *setting;
 	GHashTable *settings_hash;
 	GHashTable *secrets;
-	GError *err = NULL;
+	GError *error = NULL;
 
 	/* Got a user response, clear the NMActiveConnection destroy handler for
 	 * this dialog since this function will now take over dialog destruction.
@@ -475,18 +475,22 @@ get_pppoe_secrets_cb (GtkDialog *dialog,
 	g_object_weak_unref (G_OBJECT (info->active_connection), destroy_pppoe_dialog, info);
 
 	if (response != GTK_RESPONSE_OK) {
-		g_set_error (&err, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_SECRETS_REQUEST_CANCELED,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_SECRETS_REQUEST_CANCELED,
 		             "%s.%d (%s): canceled",
 		             __FILE__, __LINE__, __func__);
 		goto done;
 	}
 
-	setting = nm_connection_get_setting (info->connection, NM_TYPE_SETTING_PPPOE);
+	setting = nm_connection_get_setting (NM_CONNECTION (info->connection), NM_TYPE_SETTING_PPPOE);
 	pppoe_update_setting (NM_SETTING_PPPOE (setting), info);
 
 	secrets = nm_setting_to_hash (setting);
 	if (!secrets) {
-		g_set_error (&err, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
 					 "%s.%d (%s): failed to hash setting '%s'.",
 					 __FILE__, __LINE__, __func__, nm_setting_get_name (setting));
 		goto done;
@@ -499,25 +503,23 @@ get_pppoe_secrets_cb (GtkDialog *dialog,
 										   g_free, (GDestroyNotify) g_hash_table_destroy);
 
 	g_hash_table_insert (settings_hash, g_strdup (nm_setting_get_name (setting)), secrets);
-	dbus_g_method_return (info->context, settings_hash);
+	info->callback (info->connection, settings_hash, NULL, info->callback_data);
 	g_hash_table_destroy (settings_hash);
 
 	/* Save the connection back to GConf _after_ hashing it, because
 	 * saving to GConf might trigger the GConf change notifiers, resulting
 	 * in the connection being read back in from GConf which clears secrets.
 	 */
-	gconf_connection = nma_gconf_settings_get_by_connection (info->applet->gconf_settings, info->connection);
-	if (gconf_connection)
-		nma_gconf_connection_save (gconf_connection);
+	nm_settings_connection_interface_update (info->connection, update_cb, NULL);
 
 done:
-	if (err) {
-		g_warning ("%s", err->message);
-		dbus_g_method_return_error (info->context, err);
-		g_error_free (err);
+	if (error) {
+		g_warning ("%s", error->message);
+		info->callback (info->connection, NULL, error, info->callback_data);
+		g_error_free (error);
 	}
 
-	nm_connection_clear_secrets (info->connection);
+	nm_connection_clear_secrets (NM_CONNECTION (info->connection));
 	destroy_pppoe_dialog (info, NULL);
 }
 
@@ -534,10 +536,11 @@ show_password_toggled (GtkToggleButton *button, gpointer user_data)
 
 static gboolean
 pppoe_get_secrets (NMDevice *device,
-				   NMConnection *connection,
+				   NMSettingsConnectionInterface *connection,
 				   NMActiveConnection *active_connection,
 				   const char *setting_name,
-				   DBusGMethodInvocation *context,
+				   NMANewSecretsRequestedFunc callback,
+				   gpointer callback_data,
 				   NMApplet *applet,
 				   GError **error)
 {
@@ -547,13 +550,15 @@ pppoe_get_secrets (NMDevice *device,
 
 	xml = glade_xml_new (GLADEDIR "/ce-page-dsl.glade", "DslPage", NULL);
 	if (!xml) {
-		g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+		g_set_error (error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
 					 "%s.%d (%s): couldn't display secrets UI",
 		             __FILE__, __LINE__, __func__);
 		return FALSE;
 	}
 
-	info = pppoe_info_new (xml, applet, context, connection, active_connection);
+	info = pppoe_info_new (xml, applet, callback, callback_data, connection, active_connection);
 
 	/* Create the dialog */
 	info->dialog = gtk_dialog_new ();
@@ -568,7 +573,7 @@ pppoe_get_secrets (NMDevice *device,
 	                    glade_xml_get_widget (xml, "DslPage"),
 	                    TRUE, TRUE, 0);
 
-	pppoe_update_ui (connection, info);
+	pppoe_update_ui (NM_CONNECTION (connection), info);
 
 	w = glade_xml_get_widget (xml, "dsl_show_password");
 	g_signal_connect (G_OBJECT (w), "toggled", G_CALLBACK (show_password_toggled), info);
@@ -595,7 +600,8 @@ typedef struct {
 	NMApplet *applet;
 	NMActiveConnection *active_connection;
 	GtkWidget *dialog;
-	DBusGMethodInvocation *context;
+	NMANewSecretsRequestedFunc callback;
+	gpointer callback_data;
 } NM8021xInfo;
 
 static void
@@ -614,12 +620,11 @@ get_8021x_secrets_cb (GtkDialog *dialog,
 					  gpointer user_data)
 {
 	NM8021xInfo *info = user_data;
-	NMAGConfConnection *gconf_connection;
-	NMConnection *connection = NULL;
+	NMSettingsConnectionInterface *connection = NULL;
 	NMSetting *setting;
 	GHashTable *settings_hash;
 	GHashTable *secrets;
-	GError *err = NULL;
+	GError *error = NULL;
 
 	/* Got a user response, clear the NMActiveConnection destroy handler for
 	 * this dialog since this function will now take over dialog destruction.
@@ -627,7 +632,9 @@ get_8021x_secrets_cb (GtkDialog *dialog,
 	g_object_weak_unref (G_OBJECT (info->active_connection), destroy_8021x_dialog, info);
 
 	if (response != GTK_RESPONSE_OK) {
-		g_set_error (&err, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_SECRETS_REQUEST_CANCELED,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_SECRETS_REQUEST_CANCELED,
 		             "%s.%d (%s): canceled",
 		             __FILE__, __LINE__, __func__);
 		goto done;
@@ -635,35 +642,30 @@ get_8021x_secrets_cb (GtkDialog *dialog,
 
 	connection = nma_wired_dialog_get_connection (info->dialog);
 	if (!connection) {
-		g_set_error (&err, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): couldn't get connection from wired dialog.",
 		             __FILE__, __LINE__, __func__);
 		goto done;
 	}
 
-	setting = nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X);
+	setting = nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_802_1X);
 	if (!setting) {
-		g_set_error (&err, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INVALID_CONNECTION,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INVALID_CONNECTION,
 					 "%s.%d (%s): requested setting '802-1x' didn't"
 					 " exist in the connection.",
 					 __FILE__, __LINE__, __func__);
 		goto done;
 	}
 
-	if (!utils_fill_connection_certs (NM_CONNECTION (connection), &err)) {
-		g_set_error (&err, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
-					 "%s.%d (%s): failed to read connection certificates: (%d) %s.",
-					 __FILE__, __LINE__, __func__,
-					 err ? err->code : -1,
-					 err && err->message ? err->message : "(unknown)");
-		goto done;
-	}
-
 	secrets = nm_setting_to_hash (setting);
-	utils_clear_filled_connection_certs (NM_CONNECTION (connection));
-
 	if (!secrets) {
-		g_set_error (&err, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
 					 "%s.%d (%s): failed to hash setting '%s'.",
 					 __FILE__, __LINE__, __func__, nm_setting_get_name (setting));
 		goto done;
@@ -676,36 +678,35 @@ get_8021x_secrets_cb (GtkDialog *dialog,
 										   g_free, (GDestroyNotify) g_hash_table_destroy);
 
 	g_hash_table_insert (settings_hash, g_strdup (nm_setting_get_name (setting)), secrets);
-	dbus_g_method_return (info->context, settings_hash);
+	info->callback (connection, settings_hash, NULL, info->callback_data);
 	g_hash_table_destroy (settings_hash);
 
 	/* Save the connection back to GConf _after_ hashing it, because
 	 * saving to GConf might trigger the GConf change notifiers, resulting
 	 * in the connection being read back in from GConf which clears secrets.
 	 */
-	gconf_connection = nma_gconf_settings_get_by_connection (info->applet->gconf_settings, connection);
-	if (gconf_connection)
-		nma_gconf_connection_save (gconf_connection);
+	nm_settings_connection_interface_update (connection, update_cb, NULL);
 
 done:
-	if (err) {
-		g_warning ("%s", err->message);
-		dbus_g_method_return_error (info->context, err);
-		g_error_free (err);
+	if (error) {
+		g_warning ("%s", error->message);
+		info->callback (connection, NULL, error, info->callback_data);
+		g_error_free (error);
 	}
 
 	if (connection)
-		nm_connection_clear_secrets (connection);
+		nm_connection_clear_secrets (NM_CONNECTION (connection));
 
 	destroy_8021x_dialog (info, NULL);
 }
 
 static gboolean
 nm_8021x_get_secrets (NMDevice *device,
-					  NMConnection *connection,
+					  NMSettingsConnectionInterface *connection,
 					  NMActiveConnection *active_connection,
 					  const char *setting_name,
-					  DBusGMethodInvocation *context,
+					  NMANewSecretsRequestedFunc callback,
+					  gpointer callback_data,
 					  NMApplet *applet,
 					  GError **error)
 {
@@ -717,17 +718,20 @@ nm_8021x_get_secrets (NMDevice *device,
 								   g_object_ref (connection),
 								   device);
 	if (!dialog) {
-		g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+		g_set_error (error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): couldn't display secrets UI",
 		             __FILE__, __LINE__, __func__);
 		return FALSE;
 	}
 
 	info = g_malloc0 (sizeof (NM8021xInfo));
-	info->context = context;
 	info->applet = applet;
 	info->active_connection = active_connection;
 	info->dialog = dialog;
+	info->callback = callback;
+	info->callback_data = callback_data;
 
 	g_signal_connect (dialog, "response", G_CALLBACK (get_8021x_secrets_cb), info);
 
@@ -745,11 +749,12 @@ nm_8021x_get_secrets (NMDevice *device,
 
 static gboolean
 wired_get_secrets (NMDevice *device,
-				   NMConnection *connection,
+				   NMSettingsConnectionInterface *connection,
 				   NMActiveConnection *active_connection,
 				   const char *setting_name,
 				   const char **hints,
-				   DBusGMethodInvocation *context,
+				   NMANewSecretsRequestedFunc callback,
+				   gpointer callback_data,
 				   NMApplet *applet,
 				   GError **error)
 {
@@ -757,9 +762,11 @@ wired_get_secrets (NMDevice *device,
 	const char *connection_type;
 	gboolean success = FALSE;
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_CONNECTION));
 	if (!s_con) {
-		g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INVALID_CONNECTION,
+		g_set_error (error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INVALID_CONNECTION,
 		             "%s.%d (%s): Invalid connection",
 		             __FILE__, __LINE__, __func__);
 		return FALSE;
@@ -767,9 +774,24 @@ wired_get_secrets (NMDevice *device,
 
 	connection_type = nm_setting_connection_get_connection_type (s_con);
 	if (!strcmp (connection_type, NM_SETTING_WIRED_SETTING_NAME)) {
-		success = nm_8021x_get_secrets (device, connection, active_connection, setting_name, context, applet, error);
-	} else if (!strcmp (connection_type, NM_SETTING_PPPOE_SETTING_NAME))
-		success = pppoe_get_secrets (device, connection, active_connection, setting_name, context, applet, error);
+		success = nm_8021x_get_secrets (device,
+		                                connection,
+		                                active_connection,
+		                                setting_name,
+		                                callback,
+		                                callback_data,
+		                                applet,
+		                                error);
+	} else if (!strcmp (connection_type, NM_SETTING_PPPOE_SETTING_NAME)) {
+		success = pppoe_get_secrets (device,
+		                             connection,
+		                             active_connection,
+		                             setting_name,
+		                             callback,
+		                             callback_data,
+		                             applet,
+		                             error);
+	}
 
 	return success;
 }
